#!/bin/bash
# Copyright 1999-2003 Gentoo Technologies, Inc.
# Distributed under the terms of the GNU General Public License, v2 or later
# $Header: /home/cvsroot/gentoo-projects/hardened-gcc/gentoo/distrib/2.4.5/x86/hardened-gcc-arch-specific.sh,v 1.2 2003/11/09 16:41:13 pappy Exp $

source /etc/init.d/functions.sh || {
    echo "$0: Could not source /etc/init.d/functions.sh!"
    exit 1
}

HGCC_SCRIPT_EBUILD_REPLACEMENT_CONFILE="/etc/hcc.conf"

source "${HGCC_SCRIPT_EBUILD_REPLACEMENT_CONFILE}" || {
	echo "$0: Could not source ${HGCC_SCRIPT_EBUILD_REPLACEMENT_CONFILE}"
	exit 1
}

source "${HGCC_SCRIPT_EBUILD_REPLACEMENT_COMMONS}" || {
    echo "$0: Could not source ${HGCC_SCRIPT_EBUILD_REPLACEMENT_COMMONS}"
    exit 1
}

activate_transparent() {

	source "${HGCC_SCRIPT_EBUILD_REPLACEMENT_CONFILE}"

	gcc -dumpspecs > ${SPECS_GCC_CONFIG_L}

	[ "${HGCC_ENABLE_AUTO_PROPO_PREPROC_SUPPORT}" == "yes" ] && SSP_CPP_SUPPORT="${HGCC_TSPECS_AUTO_PROPO_PREPROC_SUPPORT}"
	# gcc-3.3.1 and -fstack-protector needs -fforce-addr to not make compiler goes ICEing - #30268 samba, found out by Kumba
	[ "$(gcc -dumpversion|sed 's:\.::g')" -ge "331" ] && SSP_CPP_SUPPORT="${SSP_CPP_SUPPORT} %{!yno_propolice: -fforce-addr}"
	
	# (1) add PIC definitions and -fstack-protector, disabled by either: -D__KERNEL__ -DIN_GCC -static -yet_exe
		# add PIC definesc
		SPECS_SECTION="\*cpp\:"
		REPLACESTRING="s,%(cpp_cpu),${SPECS_STD_EXCL_FRONT} %{!yet_exec: -D__PIC__ -D__pic__} ${SPECS_STD_EXCL_BACK} %(cpp_cpu),"
		eval ${SED_HELPER_FUNCTION}
		
		# add -fPIC and -fstack-protector but disabled by: -yno_propolice
		SPECS_SECTION="\*cc1\:"
		REPLACESTRING="s;%{profile:-p};${SPECS_STD_EXCL_FRONT} %{!yet_exec: -fPIC} ${SSP_CPP_SUPPORT} ${SPECS_STD_EXCL_BACK} %{profile:-p};"
		eval ${SED_HELPER_FUNCTION}
		
	# (2) modify endfile, startfile, crt1, add interp and dynamic propolice library
		# endfile modifications for PIC objects, exclusions for kernel, gcc and yet_exec
		SPECS_SECTION="\*endfile\:"
		ENDFILE_EXCL="%{!yet_exec: %{!D__KERNEL__: %{!DIN_GCC: crtendS.o%s}}} %{yet_exec: crtend.o%s} %{!yet_exec: %{D__KERNEL__: crtend.o%s}} %{!yet_exec: %{DIN_GCC: crtend.o%s}}"
		REPLACESTRING="s,%{!shared:crtend.o%s} %{shared:crtendS.o%s} crtn.o%s,%{static:crtend.o%s} %{shared:crtendS.o%s} %{!static: %{!shared: ${ENDFILE_EXCL} } } crtn.o%s,"
		eval ${SED_HELPER_FUNCTION}

		# startfile modifications for crtbegin(normal) crtbeginS(shared and etdyn) and crtbeginT(static)
		# also modifications for the propolice library
		SPECS_SECTION="\*startfile\:"
		STARTFILE_EXCL="%{!yet_exec: %{!D__KERNEL__: %{!DIN_GCC: crtbeginS.o%s}}} %{yet_exec: crtbegin.o%s} %{!yet_exec: %{D__KERNEL__: crtbegin.o%s}} %{!yet_exec: %{DIN_GCC: crtbegin.o%s}}"
		REPLACESTRING="s,%{static:crtbeginT.o%s} %{!static:%{!shared:crtbegin.o%s} %{shared:crtbeginS.o%s}},%{static:crtbeginT.o%s } %{shared:crtbeginS.o%s} %{!static: %{!shared: ${STARTFILE_EXCL} } },"
		eval ${SED_HELPER_FUNCTION}

		# set up the noCSU or CSU start1S.o in the startfile section
		SPECS_SECTION="\*startfile\:"
		STARTFILE_EXCL="%{static: crt1.o%s} %{!static: %{yet_exec: crt1.o%s} %{!yet_exec: %{D__KERNEL__: crt1.o%s}} %{!yet_exec: %{DIN_GCC: crt1.o%s}} }"
		REPLACESTRING_START="s,%{!profile:crt1.o%s},%{!profile:"
		
		HGCC_NON_CSU="${HGCC_TSPECS_STARTSECTION_CRT1S_NON_CSU} ${HGCC_TSPECS_STARTSECTION_DYNAM_INTERPR}"
		einfo "activating startfile section: ${HGCC_NON_CSU}"
		REPLACESTRING="${REPLACESTRING_START} ${STARTFILE_EXCL} ${SPECS_STD_EXCL_FRONT} ${HGCC_NON_CSU} ${SPECS_STD_EXCL_BACK} },"
		eval ${SED_HELPER_FUNCTION}
		
	# (3) modify linking to emit etdyn by default
		
		# set up pie linking or -shared linking depending in binutils - binutils 2.14.90.0.6 and higher support -pie, others -shared
		if [ $(ld -v | tr ' ' '\n' | tail -n 1) -ge "20030820" ]
		then
			einfo "activating linker section with -pie support: ${HGCC_TSPECS_LINKSECTION_PIE_ACTIVATION} ${LNK_VER_SUPPORT}"; LINK_OPTS="${HGCC_TSPECS_LINKSECTION_PIE_ACTIVATION}"
		else
			einfo "activating linker section with -shared support: ${HGCC_TSPECS_LINKSECTION_SHARED_ACTIVAT} ${LNK_VER_SUPPORT}"; LINK_OPTS="${HGCC_TSPECS_LINKSECTION_SHARED_ACTIVAT}"
		fi

		SPECS_SECTION="\*link\:"
		REPLACESTRING="s,%{static:-static}}},%{static:-static}}} ${SPECS_STD_EXCL_FRONT} ${SPECS_LINK_EXC_FRONT} ${LINK_OPTS} ${SPECS_STD_EXCL_BACK} ${SPECS_LINK_EXC_BACK},";
		eval ${SED_HELPER_FUNCTION}

	return 0
}

eval ${DOIT}

exit 0
